Attribute VB_Name = "pathfinder_title"
Public show_filename As Boolean, level As Integer
' Declarations used to force the window to stay on top of the other windows.
Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, _
                                            ByVal hWndInsertAfter As Long, _
                                            ByVal x As Long, _
                                            ByVal y As Long, _
                                            ByVal cx As Long, _
                                            ByVal cy As Long, _
                                            ByVal wFlags As Long) As Long

Public Const HWND_TOPMOST = -1
Public Const SWP_NOMOVE = &H2
Public Const SWP_NOSIZE = &H1
Global showProperty As String
Global occIndex As Integer

Sub Main()
Dim objApp As SolidEdgeFramework.Application, objModel As AssemblyDocument
Dim i As Integer, j As Integer, n As Integer

    On Error Resume Next
    Set objApp = GetObject(, "SolidEdge.Application")
    If Err Then
        Err.Clear
        MsgBox "Solid Edge must be open."
        End
    End If
    On Error GoTo 0
    If objApp.ActiveDocumentType <> igAssemblyDocument Then
        MsgBox "The active document must be an Assembly."
        End
    End If
    Set objModel = objApp.ActiveDocument

    objApp.DelayCompute = True
    Load frmProgress
    frmProgress.Show
    WindowToTop frmProgress.hWnd
    
    'Process the set
    level = 1
    Call loopStructure(objModel, level)
    Unload frmProgress
    objApp.DelayCompute = False
    Set objModel = Nothing
    Set objApp = Nothing
    End

End Sub

Private Sub loopStructure(document As Object, level As Integer)
Dim prevFileProp As String, prevFile As String
Dim currentFile As String
Dim i As Integer, n As Integer, j As Integer
Dim objLinkedDocs As solidedgeassembly.Occurrences
    If level = 1 Then
        Set objLinkedDocs = document.Occurrences
    Else
        Set objLinkedDocs = document.OccurrenceDocument.Occurrences
    End If
    With objLinkedDocs
        If .Count > 0 Then
            'Opens progress bar and determines instance number of occurence.
            'Instance number is the number that shows up after ":"
            
            n = 1 'The default value for instance number is 1.
            
            frmProgress.pb1.Min = 0
            frmProgress.pb1.Max = .Count
            
            'Loops through each occurence in the top-level assembly
            For i = occIndex To .Count
                frmProgress.pb1.Max = .Count

                currentFile = .Item(i).OccurrenceFileName
                'Show the progress form and what file is being processed.
                frmProgress.pb1.Value = i
                frmProgress.Label1.Caption = "Processing: " & currentFile
                frmProgress.Refresh
                'Checks current occurence file name against previous occurences to set
                'number of instances for the current occurence.
                For j = i - 1 To 1 Step -1
                    prevFileProp = .Item(j).Name
                    prevFile = .Item(j).OccurrenceFileName
                    If LCase(currentFile) = LCase(prevFile) Then
                    'If the current file name matches a previous file name the instance #
                    'is set to 1 plus the instance number of the previous occurence
                        n = CInt(Mid(prevFileProp, InStr(prevFileProp, ":") + 1)) + 1
                        Exit For
                    End If
                Next j
                
                'I put the name of the piece
                Call changeProp(.Item(i), n, showProperty)

                If UCase(Right(.Item(i).OccurrenceFileName, 3)) = "ASM" Then
                    Call loopStructure(.Item(i), level + 1)
                End If
            Next i
        End If
    End With
End Sub

'-----------------------------------------------
'changeProp:
'-----------------------------------------------
Sub changeProp(document As Occurrence, ndoc As Integer, showProperty As String)
    With document
        Select Case showProperty
            Case "File Name"
                .Name = removePath(.OccurrenceFileName) & ":" & ndoc
            Case "Title & Part Number"
                If Trim(.OccurrenceDocument.SummaryInfo.Title) = "" Then
                    .Name = "*** " & removePath(.OccurrenceFileName) & ":" & ndoc
                Else
                    .Name = .OccurrenceDocument.SummaryInfo.Title & " (" & _
                        .OccurrenceDocument.SummaryInfo.DocumentNumber & ")" & ":" & ndoc
                End If
            Case "Reference Part Number"
                If Trim(.OccurrenceDocument.SummaryInfo.DocumentNumber) = "" Then
                    .Name = "*** " & removePath(.OccurrenceFileName) & ":" & ndoc
                Else
                    .Name = .OccurrenceDocument.SummaryInfo.DocumentNumber & ":" & ndoc
                End If
            Case "Material & Part Number"
                'Must use Property Sets for Material because it is not in
                ' .OccurrenceDocument.SummaryInfo
                Dim objpropsets As SolidEdgeFileProperties.PropertySets
                Dim objProperties As SolidEdgeFileProperties.Properties
                Dim Prop As Object
                Set objpropsets = CreateObject("SolidEdge.FileProperties")
                On Error Resume Next
                    Call objpropsets.Open(document.OccurrenceFileName, False)
                    If Err Then
                        Err.Clear
                        MsgBox "File " & document.OccurrenceFileName & " must be closed to show material."
                        Exit Sub
                    End If
                On Error GoTo 0
                
                Set objProperties = objpropsets.Item("MechanicalModeling")
                Set Prop = objProperties.Item("Material")
                
                If Trim(Prop.Value) = "" Then
                    .Name = "*** " & removePath(.OccurrenceFileName) & ":" & ndoc
                Else
                    .Name = Prop.Value & " (" & _
                        .OccurrenceDocument.SummaryInfo.DocumentNumber & ")" & ":" & ndoc
                End If
                
        End Select
    End With
End Sub


'------------------------------------------------------------------
'removePath: Removes the path information from the file name and
'            leaves just the file name and extension.
'------------------------------------------------------------------
Function removePath(completePath As String) As String
    removePath = Mid(completePath, findChar(completePath, "\") + 1)
End Function


'------------------------------------------------------------------
' findChar: Finds the location in a string of the last specified
'           character.
'------------------------------------------------------------------
Function findChar(text As String, character As String) As Integer
Dim i As Integer
    findChar = 0
    'The following loop works backwards and starts at the 5th to last
    'character, this is do to the assumption that the filename including
    'extension is at least 5 characters long (a.par = char).  This saves time.
    For i = Len(text) - 5 To 1 Step -1
        If Mid(text, i, 1) = character Then
            findChar = i
            Exit For
        End If
    Next i
End Function

' Forces a window to the top and keeps it on top.
Public Sub WindowToTop(hWnd As Long)
    Call SetWindowPos(hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE)
End Sub



